<?php
namespace Drupal\miniorange_saml_idp;

class MetadataReader{

	private $serviceProviders;

	public function __construct(\DOMNode $xml = NULL) {

		$this->serviceProviders = array();
    
		$entityDescriptors = Utilities::xpQuery($xml, './saml_metadata:EntityDescriptor');

        foreach ($entityDescriptors as $entityDescriptor) {
            $SPSSODescriptor = Utilities::xpQuery($entityDescriptor, './saml_metadata:SPSSODescriptor');
            if(isset($SPSSODescriptor) && !empty($SPSSODescriptor)){
				array_push($this->serviceProviders, new ServiceProviders($entityDescriptor));
			}
		}
	}

	public function getServiceProviders(){
		return $this->serviceProviders;
	}
}

class ServiceProviders{

    private $entityID;
    private $acsURL;
    private $assertionsSigned;
    private $logoutDetails;
    private $signingCertificate;
    private $encryptionCertificate;
    private $signedRequest;

    public function __construct(\DOMElement $xml = NULL) {

        $this->logoutDetails = array();
        $this->signingCertificate = array();
        $this->encryptionCertificate = array();

        if ($xml->hasAttribute('entityID')) {
            $this->entityID = $xml->getAttribute('entityID');
        }

        if($xml->hasAttribute('WantAuthnRequestsSigned')){
            $this->signedRequest = $xml->getAttribute('WantAuthnRequestsSigned');
        }

        $SPSSODescriptor = Utilities::xpQuery($xml, './saml_metadata:SPSSODescriptor');

        if (count($SPSSODescriptor) > 1) {
            throw new Exception('More than one <SPSSODescriptor> in <EntityDescriptor>.');
        } elseif (empty($SPSSODescriptor)) {
            throw new Exception('Missing required <SPSSODescriptor> in <EntityDescriptor>.');
        }

        
        $this->parseAcsURL($SPSSODescriptor);
        $this->assertionsSigned($SPSSODescriptor);
        $this->parseLogoutUrl($SPSSODescriptor);
        $this->parsex509Certificate($SPSSODescriptor);
    }

    private function parseAcsURL($SPSSODescriptor){

        $AssertionConsumerService = Utilities::xpQuery($SPSSODescriptor[0], './saml_metadata:AssertionConsumerService');
        foreach ($AssertionConsumerService as $sign) {
            if($sign->hasAttribute('Location')){
                $this->acsURL = $sign->getAttribute('Location');
            }
        }
    }

    private function assertionsSigned($SPSSODescriptor){

        foreach ($SPSSODescriptor as $sign) {
            if($sign->hasAttribute('WantAssertionsSigned')){
                $this->assertionsSigned = $sign->getAttribute('WantAssertionsSigned');
            }
        }
    }

    private function parseLogoutURL($xml){
        $sloServices = Utilities::xpQuery($xml[0], './saml_metadata:SingleLogoutService');
        foreach ($sloServices as $sloService) {
            $binding = str_replace("urn:oasis:names:tc:SAML:2.0:bindings:","",$sloService->getAttribute('Binding'));
            $this->logoutDetails = array_merge($this->logoutDetails, array($binding => $sloService->getAttribute('Location')));
        }
    }

    
    private function parsex509Certificate($xml){
        $X_cert = Utilities::xpQuery($xml[0], './saml_metadata:KeyDescriptor');
        foreach ( $X_cert as $KeyDescriptorNode ) {
            if($KeyDescriptorNode->hasAttribute('use')){
                if($KeyDescriptorNode->getAttribute('use')=='encryption'){
                    $this->parseEncryptionCertificate($KeyDescriptorNode);
                }else{
                    $this->parseSigningCertificate($KeyDescriptorNode);
                }
            }else{
                $this->parseSigningCertificate($KeyDescriptorNode);
            }
        }
    }

    private function parseSigningCertificate($xml){
        $certNode = Utilities::xpQuery($xml, './ds:KeyInfo/ds:X509Data/ds:X509Certificate');
        $certData = trim($certNode[0]->textContent);
        $certData = str_replace(array ( "\r", "\n", "\t", ' '), '', $certData);
        if(!empty($certNode)){
            $this->signingCertificate = Utilities::sanitize_certificate( $certData );
        }
    }


    private function parseEncryptionCertificate($xml){
        $certNode = Utilities::xpQuery($xml, './ds:KeyInfo/ds:X509Data/ds:X509Certificate');
        $certData = trim($certNode[0]->textContent);
        $certData = str_replace(array ( "\r", "\n", "\t", ' '), '', $certData);
        if(!empty($certNode))
            $this->encryptionCertificate = Utilities::sanitize_certificate( $certData );
    }

    public function getEntityID(){
        return $this->entityID;
    }

    public function getAcsURL(){
        return $this->acsURL;
    }

    public function getAssertionsSigned(){
        return $this->assertionsSigned;
    }

    public function getLogoutURL($binding){
        return isset($this->logoutDetails[$binding])? $this->logoutDetails[$binding]: '';
    }

    public function getLogoutDetails(){
        return $this->logoutDetails;
    }

    public function getSigningCertificate(){
        return $this->signingCertificate;
    }

    public function getEncryptionCertificate(){
        return $this->encryptionCertificate[0];
    }

    public function isRequestSigned(){
        return $this->signedRequest;
    } 
}
